# Copyright (c) 2010 gocept gmbh & co. kg
# See also LICENSE.txt

import ConfigParser
import datetime
import icalendar
import optparse
import os
import pytz
import sys


class Contact(object):
    """A staff contact."""

    def __init__(self, name, email, mobile, pager):
        self.name = name
        self.email = email
        self.mobile = mobile
        self.pager = pager


class Staff(object):

    def __init__(self, filename):
        self.cfg = ConfigParser.ConfigParser()
        self.cfg.read(filename)
        self.tz = pytz.timezone(self.cfg.get('general', 'timezone'))

        self.contacts = {}
        for section in self.cfg.sections():
            if not section.startswith('contact-'):
                continue
            contact = Contact(
                section.replace('contact-', ''),
                self.cfg.get(section, 'email'),
                self.cfg.get(section, 'mobile'),
                self.cfg.get(section, 'pager'))
            self.contacts[contact.name] = contact

    def assignments(self, day):
        """Return assignments, first one is primary, others are secondary"""
        contacts = []
        # Try parsing by format version 1
        try:
            for key, value in self.cfg.items(day.strftime('%Y-W%W') +
                                             '-%s' % day.isoweekday()):
                contacts.append((key, int(value)))
        except ConfigParser.NoSectionError:
            try:
                for key, value in self.cfg.items(day.strftime('%Y-W%W')):
                    contacts.append((key, int(value)))
            except ConfigParser.NoSectionError:
                pass
        contacts = [x[0] for x in sorted(contacts, key=lambda x:x[1])]
        # Try parsing by format version 2
        try:
            value = self.cfg.get('%s-%s' % (day.year, day.month), str(day.day))
            if contacts:
                raise RuntimeError(
                    'Duplicate defition of day %s in format version 1 and 2.'
                    % day)
            if value == '...':
                # Use contact IDs from day before
                contacts.extend(x.name for x in self.assignments(day - datetime.timedelta(days=1)))
            else:
                contacts.extend(value.split(','))
        except (ConfigParser.NoSectionError, ConfigParser.NoOptionError):
            pass
        # Convert contact names to contact objects
        return [self.contacts[x.strip()] for x in contacts]

    def events(self, begin=None, end=None):
        if begin is None:
            begin = datetime.date.today() - datetime.timedelta(days=60)
        if end is None:
            end = datetime.date.today() + datetime.timedelta(days=180)

        current = begin
        while current <= end:
            primary = None
            secondary = []
            for assignment in self.assignments(current):
                if primary is None:
                    primary = assignment.name
                else:
                    secondary.append(assignment.name)
            if primary is not None:
                event = icalendar.Event()
                summary = primary
                if secondary:
                    summary += ' (%s)' % (', '.join(secondary),)
                event.add('summary', summary)
                event.add('dtstart', current)
                event.add('dtend', current)
                event.add('dtstamp', datetime.datetime.now(self.tz))
                yield event
            current += datetime.timedelta(days=1)

    def statistics(self):
        stats = {}
        year = datetime.date.today().year
        day = datetime.date(year=year, month=1, day=1)
        while day.year == year:
            for i, assignment in enumerate(self.assignments(day)):
                primary, secondary, points = stats.setdefault(assignment.name,
                                                      (0, 0, 0))
                if not i:
                    primary += 1
                else:
                    secondary += 1
                stats[assignment.name] = (primary, secondary,
                                          primary+(secondary*1/3.0))
            day += datetime.timedelta(days=1)
        return stats


def create_ical_for_staff():
    parser = optparse.OptionParser()
    parser.add_option('-o', '--output', default=None)
    parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
    options, args = parser.parse_args()
    conf = ConfigParser.ConfigParser()
    conf.read([os.path.expanduser(options.config)])

    cal = icalendar.Calendar()
    cal.add('prodid', '-//gocept.support//gocept//')

    staff = Staff(conf.get('main', 'staff'))
    for event in staff.events():
        cal.add_component(event)

    if options.output is None:
        f = sys.stdout
    else:
        f = open(options.output, 'w')
    f.write(cal.as_string())


def generate_month(month, out=sys.stdout):
    year, month = [int(x) for x in month.split('-')]
    now = datetime.date(year, month, day=1)
    print >> out, "[%s-%s]" % (year, month)
    while now.month == month:
        if now.isoweekday() == 1:
            print >> out, "\n# Monday"
        print >> out, "%s = ..." % now.day
        now += datetime.timedelta(days=1)


def generate_month_main():
    parser = optparse.OptionParser()
    options, args = parser.parse_args()
    generate_month(args[0])


def get_statistics():
    parser = optparse.OptionParser()
    parser.add_option('-o', '--output', default=None)
    parser.add_option('-c', '--config', default='~/.gocept-support.cfg')
    options, args = parser.parse_args()
    conf = ConfigParser.ConfigParser()
    conf.read([os.path.expanduser(options.config)])

    staff = Staff(conf.get('main', 'staff'))

    print "Name".ljust(20), "1st  ", "2nd  ", "Total"
    for supporter, days in sorted(staff.statistics().items(),
                                  key=lambda x:x[1][2], reverse=True):
        print supporter.ljust(20), str(days[0]).ljust(5), str(days[1]).ljust(5), '%0.2f' % days[2]
